@@ -96,6 +96,12 @@
 GST_DEBUG_CATEGORY_STATIC (gnomevfssrc_debug);
 #define GST_CAT_DEFAULT gnomevfssrc_debug
 
+static GstElementDetails gst_gnome_vfs_src_details =
+GST_ELEMENT_DETAILS ("GnomeVFS Source",
+    "Source/File",
+    "Read from any GnomeVFS-supported file",
+    "Bastien Nocera <hadess@hadess.net>\n"
+    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
 
 static GStaticMutex count_lock = G_STATIC_MUTEX_INIT;
 static gint ref_count = 0;
@@ -183,12 +189,6 @@ static void
 gst_gnome_vfs_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  static GstElementDetails gst_gnome_vfs_src_details =
-      GST_ELEMENT_DETAILS ("GnomeVFS Source",
-      "Source/File",
-      "Read from any GnomeVFS-supported file",
-      "Bastien Nocera <hadess@hadess.net>\n"
-      "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&srctemplate));
@@ -314,10 +314,20 @@ gst_gnome_vfs_src_finalize (GObject * object)
     src->uri = NULL;
   }
 
-  if (src->uri_name) {
-    g_free (src->uri_name);
-    src->uri_name = NULL;
-  }
+  g_free (src->uri_name);
+  src->uri_name = NULL;
+
+  g_free (src->iradio_name);
+  src->iradio_name = NULL;
+
+  g_free (src->iradio_genre);
+  src->iradio_genre = NULL;
+
+  g_free (src->iradio_url);
+  src->iradio_url = NULL;
+
+  g_free (src->iradio_title);
+  src->iradio_title = NULL;
 
   g_mutex_free (src->audiocast_udpdata_mutex);
   g_mutex_free (src->audiocast_queue_mutex);
@@ -384,12 +394,13 @@ gst_gnome_vfs_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstGnomeVFSSrc *src;
-  gchar cwd[PATH_MAX];
 
   src = GST_GNOME_VFS_SRC (object);
 
   switch (prop_id) {
-    case ARG_LOCATION:
+    case ARG_LOCATION:{
+      const gchar *new_location;
+
       /* the element must be stopped or paused in order to do this */
       if (GST_STATE (src) == GST_STATE_PLAYING ||
           GST_STATE (src) == GST_STATE_PAUSED)
@@ -404,25 +415,13 @@ gst_gnome_vfs_src_set_property (GObject * object, guint prop_id,
         src->uri_name = NULL;
       }
 
-      if (g_value_get_string (value)) {
-        const gchar *location = g_value_get_string (value);
-
-        if (!strchr (location, ':')) {
-          gchar *newloc = gnome_vfs_escape_path_string (location);
-
-          if (*newloc == '/')
-            src->uri_name = g_strdup_printf ("file://%s", newloc);
-          else
-            src->uri_name =
-                g_strdup_printf ("file://%s/%s", getcwd (cwd, PATH_MAX),
-                newloc);
-          g_free (newloc);
-        } else
-          src->uri_name = g_strdup (location);
-
+      new_location = g_value_get_string (value);
+      if (new_location) {
+        src->uri_name = gst_gnome_vfs_location_to_uri_string (new_location);
         src->uri = gnome_vfs_uri_new (src->uri_name);
       }
       break;
+    }
     case ARG_HANDLE:
       if (GST_STATE (src) == GST_STATE_NULL ||
           GST_STATE (src) == GST_STATE_READY) {
@@ -872,6 +871,8 @@ gst_gnome_vfs_src_pop_callbacks (GstGnomeVFSSrc * src)
       (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS);
   gnome_vfs_module_callback_pop
       (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS);
+
+  src->http_callbacks_pushed = FALSE;
 }
 
 static void
@@ -942,6 +943,7 @@ gst_gnome_vfs_src_get_icy_metadata (GstGnomeVFSSrc * src)
   }
 
   g_strfreev (tags);
+  g_free (data);
 }
 
 /* end of icecast/audiocast metadata extraction support code */
