@@ -30,24 +30,26 @@
  * </para>
  * <para>
  * The theora codec internally only supports encoding of images that are a
- * multiple of 16 pixls in both X and Y direction. It is however perfectly possible 
- * to encode images with other dimensions because an arbitrary rectangular cropping
- * region can be set up. This element will automatically set up a correct cropping
- * region if the dimensions are not multiples of 16 pixels. The "border" and "center" 
- * properties control how this cropping region will be set up. 
+ * multiple of 16 pixels in both X and Y direction. It is however perfectly
+ * possible to encode images with other dimensions because an arbitrary
+ * rectangular cropping region can be set up. This element will automatically
+ * set up a correct cropping region if the dimensions are not multiples of 16
+ * pixels. The "border" and "center" properties control how this cropping
+ * region will be set up.
  * </para>
  * <para>
- * To control the quality of the encoding, the "bitrate" and "quality" properties can
- * be used. These two properties are mutualy exclusive. Setting the bitrate property
- * will produce a constant bitrate (CBR) stream while setting the quality property
- * will produce a variable bitrate (VBR) stream.
+ * To control the quality of the encoding, the "bitrate" and "quality"
+ * properties can be used. These two properties are mutualy exclusive. Setting
+ * the bitrate property will produce a constant bitrate (CBR) stream while
+ * setting the quality property will produce a variable bitrate (VBR) stream.
  * </para>
  * <title>Example pipeline</title>
  * <programlisting>
  * gst-launch -v videotestsrc num-buffers=1000 ! theoraenc ! oggmux ! filesink location=videotestsrc.ogg
  * </programlisting>
  * This example pipeline will encode a test video source to theora muxed in an
- * ogg container. Refer to the theoradec documentation to decode the create stream.
+ * ogg container. Refer to the theoradec documentation to decode the create
+ * stream.
  * </refsect2>
  *
  * Last reviewed on 2006-03-01 (0.10.4)
@@ -149,12 +151,11 @@ granulepos_to_timestamp (GstTheoraEnc * theoraenc, ogg_int64_t granulepos)
       theoraenc->info.fps_numerator);
 }
 
-static GstElementDetails theora_enc_details = {
-  "TheoraEnc",
-  "Codec/Encoder/Video",
-  "encode raw YUV video to a theora stream",
-  "Wim Taymans <wim@fluendo.com>",
-};
+static GstElementDetails theora_enc_details =
+GST_ELEMENT_DETAILS ("Theora video encoder",
+    "Codec/Encoder/Video",
+    "encode raw YUV video to a theora stream",
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate theora_enc_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
