@@ -30,9 +30,9 @@ G_BEGIN_DECLS
 #define GST_TYPE_RING_BUFFER             (gst_ring_buffer_get_type())
 #define GST_RING_BUFFER(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_RING_BUFFER,GstRingBuffer))
 #define GST_RING_BUFFER_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_RING_BUFFER,GstRingBufferClass))
-#define GST_RING_BUFFER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_RING_BUFFER, GstRingBufferClass))
+#define GST_RING_BUFFER_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_RING_BUFFER, GstRingBufferClass))
 #define GST_IS_RING_BUFFER(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_RING_BUFFER))
-#define GST_IS_RING_BUFFER_CLASS(obj)     (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RING_BUFFER))
+#define GST_IS_RING_BUFFER_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RING_BUFFER))
 
 typedef struct _GstRingBuffer GstRingBuffer;
 typedef struct _GstRingBufferClass GstRingBufferClass;
@@ -199,6 +199,23 @@ struct _GstRingBufferSpec
 #define GST_RING_BUFFER_SIGNAL(buf)   (g_cond_signal (GST_RING_BUFFER_GET_COND (buf)))
 #define GST_RING_BUFFER_BROADCAST(buf)(g_cond_broadcast (GST_RING_BUFFER_GET_COND (buf)))
 
+/**
+ * GstRingBuffer:
+ * @cond: used to signal start/stop/pause/resume actions
+ * @open: boolean indicating that the ringbuffer is open
+ * @acquired: boolean indicating that the ringbuffer is acquired
+ * @data: data in the ringbuffer
+ * @spec: format and layout of the ringbuffer data
+ * @segstate: status of each segment in the ringbuffer (unused)
+ * @samples_per_seg: number of samples in one segment
+ * @empty_seg: pointer to memory holding one segment of silence samples
+ * @state: state of the buffer
+ * @segdone: readpointer in the ringbuffer
+ * @segbase: segment corresponding to segment 0 (unused)
+ * @waiting: is a reader or writer waiting for a free segment
+ *
+ * The ringbuffer base class structure.
+ */
 struct _GstRingBuffer {
   GstObject              object;
 
@@ -209,14 +226,14 @@ struct _GstRingBuffer {
   GstBuffer             *data;
   GstRingBufferSpec      spec;
   GstRingBufferSegState *segstate;
-  gint                   samples_per_seg;     /* number of samples per segment */
+  gint                   samples_per_seg;
   guint8                *empty_seg;
 
   /*< public >*/ /* ATOMIC */
-  gint                   state;         /* state of the buffer */
-  gint                   segdone;       /* number of segments processed since last start */
-  gint                   segbase;       /* segment corresponding to segment 0 */
-  gint                   waiting;       /* when waiting for a segment to be freed */
+  gint                   state;
+  gint                   segdone;
+  gint                   segbase;
+  gint                   waiting;
 
   /*< private >*/
   GstRingBufferCallback  callback;
@@ -226,32 +243,42 @@ struct _GstRingBuffer {
   union {
     struct {
       gboolean           flushing;
+      /* ATOMIC */
+      gint               may_start;
     } ABI;
     /* adding + 0 to mark ABI change to be undone later */
     gpointer _gst_reserved[GST_PADDING + 0];
   } abidata;
 };
 
+/**
+ * GstRingBufferClass:
+ * @open_device:  open the device, don't set any params or allocate anything
+ * @acquire: allocate the resources for the ringbuffer using the given spec
+ * @release: free resources of the ringbuffer
+ * @close_device: close the device
+ * @start: start processing of samples
+ * @pause: pause processing of samples
+ * @resume: resume processing of samples after pause
+ * @stop: stop processing of samples
+ * @delay: get number of samples queued in device
+ *
+ * The vmethods that subclasses can override to implement the ringbuffer.
+ */
 struct _GstRingBufferClass {
   GstObjectClass parent_class;
 
   /*< public >*/
-  /* just open the device, don't set any params or allocate anything */
   gboolean     (*open_device)  (GstRingBuffer *buf);
-  /* allocate the resources for the ringbuffer using the given specs */
   gboolean     (*acquire)      (GstRingBuffer *buf, GstRingBufferSpec *spec);
-  /* free resources of the ringbuffer */
   gboolean     (*release)      (GstRingBuffer *buf);
-  /* close the device */
   gboolean     (*close_device) (GstRingBuffer *buf);
 
-  /* playback control */
   gboolean     (*start)        (GstRingBuffer *buf);
   gboolean     (*pause)        (GstRingBuffer *buf);
   gboolean     (*resume)       (GstRingBuffer *buf);
   gboolean     (*stop)         (GstRingBuffer *buf);
 
-  /* number of samples queued in device */
   guint        (*delay)        (GstRingBuffer *buf);
 
   /*< private >*/
@@ -310,6 +337,9 @@ gboolean        gst_ring_buffer_prepare_read    (GstRingBuffer *buf, gint *segme
 void            gst_ring_buffer_clear           (GstRingBuffer *buf, gint segment);
 void            gst_ring_buffer_advance         (GstRingBuffer *buf, guint advance);
 
+void            gst_ring_buffer_may_start       (GstRingBuffer *buf, gboolean allowed);
+
+
 G_END_DECLS
 
 #endif /* __GST_RING_BUFFER_H__ */
