@@ -33,7 +33,7 @@
  * </para>
  * <para>
  * <programlisting>
- * gst-launch audiotestsrc wave=2 freq=200 ! audioconvert ! tee name=t ! alsasink t. ! libvisual_lv_scope ! ffmpegcolorspace ! xvimagesink
+ * gst-launch audiotestsrc wave=2 freq=200 ! audioconvert ! tee name=t ! queue ! alsasink t. ! queue ! libvisual_lv_scope ! ffmpegcolorspace ! xvimagesink
  * </programlisting>
  * In this example a saw wave is generated. The wave is shown using a
  * scope visualizer from libvisual, allowing you to visually verify that
@@ -53,14 +53,22 @@
 
 #include "gstaudiotestsrc.h"
 
+
+#ifndef M_PI
+#define M_PI  3.14159265358979323846
+#endif
+
+#ifndef M_PI_2
+#define M_PI_2  1.57079632679489661923
+#endif
+
 #define M_PI_M2 ( M_PI + M_PI )
 
-GstElementDetails gst_audio_test_src_details = {
-  "Audio test source",
-  "Source/Audio",
-  "Creates audio test signals of given frequency and volume",
-  "Stefan Kost <ensonic@users.sf.net>"
-};
+static GstElementDetails gst_audio_test_src_details =
+GST_ELEMENT_DETAILS ("Audio test source",
+    "Source/Audio",
+    "Creates audio test signals of given frequency and volume",
+    "Stefan Kost <ensonic@users.sf.net>");
 
 
 enum
@@ -107,7 +115,7 @@ gst_audiostestsrc_wave_get_type (void)
     {0, NULL, NULL},
   };
 
-  if (!audiostestsrc_wave_type) {
+  if (G_UNLIKELY (audiostestsrc_wave_type == 0)) {
     audiostestsrc_wave_type = g_enum_register_static ("GstAudioTestSrcWave",
         audiostestsrc_waves);
   }
@@ -285,6 +293,7 @@ gst_audio_test_src_query (GstBaseSrc * basesrc, GstQuery * query)
       break;
     }
     default:
+      res = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
       break;
   }
 
@@ -506,7 +515,7 @@ gst_audio_test_src_create_sine_table (GstAudioTestSrc * src, gint16 * samples)
     if (src->accumulator >= M_PI_M2)
       src->accumulator -= M_PI_M2;
 
-    samples[i] = (gint16) src->wave_table[(gint) (src->accumulator * scl)];
+    samples[i] = src->wave_table[(gint) (src->accumulator * scl)];
   }
 }
 
@@ -626,6 +635,7 @@ static GstFlowReturn
 gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     guint length, GstBuffer ** buffer)
 {
+  GstFlowReturn res;
   GstAudioTestSrc *src;
   GstBuffer *buf;
   GstClockTime next_time;
@@ -636,6 +646,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
   if (src->eos_reached)
     return GST_FLOW_UNEXPECTED;
 
+  /* example for tagging generated data */
   if (!src->tags_pushed) {
     GstTagList *taglist;
     GstEvent *event;
@@ -650,6 +661,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->tags_pushed = TRUE;
   }
 
+  /* check for eos */
   if (src->check_seek_stop &&
       (src->n_samples_stop > src->n_samples) &&
       (src->n_samples_stop < src->n_samples + src->samples_per_buffer)
@@ -663,15 +675,17 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->generate_samples_per_buffer = src->samples_per_buffer;
     n_samples = src->n_samples + src->samples_per_buffer;
   }
-  next_time = n_samples * GST_SECOND / src->samplerate;
+  next_time = gst_util_uint64_scale (n_samples, GST_SECOND,
+      (guint64) src->samplerate);
 
-  buf =
-      gst_buffer_new_and_alloc (src->generate_samples_per_buffer *
-      sizeof (gint16));
-  gst_buffer_set_caps (buf, GST_PAD_CAPS (basesrc->srcpad));
+  /* allocate a new buffer suitable for this pad */
+  if ((res = gst_pad_alloc_buffer (basesrc->srcpad, src->n_samples,
+              src->generate_samples_per_buffer * sizeof (gint16),
+              GST_PAD_CAPS (basesrc->srcpad), &buf)) != GST_FLOW_OK) {
+    return res;
+  }
 
   GST_BUFFER_TIMESTAMP (buf) = src->timestamp_offset + src->running_time;
-  GST_BUFFER_OFFSET (buf) = src->n_samples;
   GST_BUFFER_OFFSET_END (buf) = n_samples;
   GST_BUFFER_DURATION (buf) = next_time - src->running_time;
 
@@ -754,6 +768,9 @@ gst_audio_test_src_get_property (GObject * object, guint prop_id,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  /* initialize gst controller library */
+  gst_controller_init (NULL, NULL);
+
   return gst_element_register (plugin, "audiotestsrc",
       GST_RANK_NONE, GST_TYPE_AUDIO_TEST_SRC);
 }
