@@ -33,6 +33,13 @@
 GST_DEBUG_CATEGORY_STATIC (sub_parse_debug);
 #define GST_CAT_DEFAULT sub_parse_debug
 
+static GstElementDetails sub_parse_details =
+GST_ELEMENT_DETAILS ("Subtitle parser",
+    "Codec/Parser/Subtitle",
+    "Parses subtitle (.sub) files into text streams",
+    "Gustavo J. A. M. Carneiro <gjc@inescporto.pt>\n"
+    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
+
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -88,13 +95,6 @@ static void
 gst_sub_parse_base_init (GstSubParseClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  static GstElementDetails sub_parse_details = {
-    "Subtitle parsers",
-    "Codec/Parser/Subtitle",
-    "Parses subtitle (.sub) files into text streams",
-    "Gustavo J. A. M. Carneiro <gjc@inescporto.pt>\n"
-        "Ronald S. Bultje <rbultje@ronald.bitfreak.net>"
-  };
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_templ));
@@ -124,7 +124,7 @@ gst_sub_parse_class_init (GstSubParseClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   object_class->dispose = gst_sub_parse_dispose;
 
@@ -230,38 +230,45 @@ beach:
 static gchar *
 convert_encoding (GstSubParse * self, const gchar * str, gsize len)
 {
-  gsize bytes_read, bytes_written;
-  gchar *rv;
-  GString *converted;
+  const gchar *encoding;
+  GError *err = NULL;
+  gchar *ret;
 
-  converted = g_string_new (NULL);
-  while (len) {
-#ifndef GST_DISABLE_GST_DEBUG
-    gchar *dbg = g_strndup (str, len);
-
-    GST_DEBUG ("Trying to convert '%s'", dbg);
-    g_free (dbg);
-#endif
-
-    rv = g_locale_to_utf8 (str, len, &bytes_read, &bytes_written, NULL);
-    if (rv) {
-      g_string_append_len (converted, rv, bytes_written);
-      g_free (rv);
-
-      len -= bytes_read;
-      str += bytes_read;
+  if (self->valid_utf8) {
+    if (g_utf8_validate (str, len, NULL)) {
+      GST_LOG_OBJECT (self, "valid UTF-8, no conversion needed");
+      return g_strndup (str, len);
     }
-    if (len) {
-      /* conversion error ocurred => skip one char */
-      len--;
-      str++;
-      g_string_append_c (converted, '?');
+    GST_INFO_OBJECT (self, "invalid UTF-8!");
+    self->valid_utf8 = FALSE;
+  }
+
+  encoding = g_getenv ("GST_SUBTITLE_ENCODING");
+  if (encoding == NULL || *encoding == '\0') {
+    /* if local encoding is UTF-8 and no encoding specified
+     * via the environment variable, assume ISO-8859-15 */
+    if (g_get_charset (&encoding)) {
+      encoding = "ISO-8859-15";
     }
   }
-  rv = converted->str;
-  g_string_free (converted, FALSE);
-  GST_DEBUG ("Converted to '%s'", rv);
-  return rv;
+
+  ret = g_convert_with_fallback (str, len, "UTF-8", encoding, "*", NULL,
+      NULL, &err);
+
+  if (err) {
+    GST_WARNING_OBJECT (self, "could not convert string from '%s' to UTF-8: %s",
+        encoding, err->message);
+    g_error_free (err);
+
+    /* invalid input encoding, fall back to ISO-8859-15 (always succeeds) */
+    ret = g_convert_with_fallback (str, len, "UTF-8", "ISO-8859-15", "*",
+        NULL, NULL, NULL);
+  }
+
+  GST_LOG_OBJECT (self, "successfully converted %d characters from %s to UTF-8"
+      "%s", len, encoding, (err) ? " , using ISO-8859-15 as fallback" : "");
+
+  return ret;
 }
 
 static gchar *
@@ -833,6 +840,7 @@ gst_sub_parse_change_state (GstElement * element, GstStateChange transition)
       /* format detection will init the parser state */
       self->offset = self->next_offset = 0;
       self->parser_type = GST_SUB_PARSE_FORMAT_UNKNOWN;
+      self->valid_utf8 = TRUE;
       break;
     default:
       break;
