@@ -15,6 +15,7 @@ static gboolean accurate_seek = FALSE;
 static gboolean keyframe_seek = FALSE;
 static gboolean loop_seek = FALSE;
 static gboolean flush_seek = TRUE;
+static gboolean scrub = FALSE;
 
 static GstElement *pipeline;
 static gint64 position;
@@ -41,11 +42,14 @@ static gulong changed_id;
 //#define VSINK "cacasink"
 
 //#define UPDATE_INTERVAL 500
-#define UPDATE_INTERVAL 100
+//#define UPDATE_INTERVAL 100
+#define UPDATE_INTERVAL 10
 
 /* number of milliseconds to play for after a seek */
-//#define SCRUB_TIME 250
-//#define SCRUB
+#define SCRUB_TIME 100
+
+/* seek timeout */
+#define SEEK_TIMEOUT 40 * GST_MSECOND
 
 #define THREAD
 #define PAD_SEEK
@@ -997,16 +1001,15 @@ update_scale (gpointer data)
 
 static void do_seek (GtkWidget * widget);
 
-#ifdef SCRUB
 static gboolean
 end_scrub (GtkWidget * widget)
 {
+  GST_DEBUG ("end scrub, PAUSE");
   gst_element_set_state (pipeline, GST_STATE_PAUSED);
   seek_timeout_id = 0;
 
   return FALSE;
 }
-#endif
 
 static gboolean
 send_event (GstEvent * event)
@@ -1074,8 +1077,7 @@ do_seek (GtkWidget * widget)
   if (res) {
     if (flush_seek) {
       gst_pipeline_set_new_stream_time (GST_PIPELINE (pipeline), 0);
-      gst_element_get_state (GST_ELEMENT (pipeline), NULL, NULL,
-          50 * GST_MSECOND);
+      gst_element_get_state (GST_ELEMENT (pipeline), NULL, NULL, SEEK_TIMEOUT);
     }
   } else
     g_print ("seek failed\n");
@@ -1084,23 +1086,24 @@ do_seek (GtkWidget * widget)
 static void
 seek_cb (GtkWidget * widget)
 {
-#ifdef SCRUB
   /* If the timer hasn't expired yet, then the pipeline is running */
-  if (seek_timeout_id != 0) {
+  if (scrub && seek_timeout_id != 0) {
+    GST_DEBUG ("do scrub seek, PAUSED");
     gst_element_set_state (pipeline, GST_STATE_PAUSED);
   }
-#endif
 
+  GST_DEBUG ("do seek");
   do_seek (widget);
 
-#ifdef SCRUB
-  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  if (scrub) {
+    GST_DEBUG ("do scrub seek, PLAYING");
+    gst_element_set_state (pipeline, GST_STATE_PLAYING);
 
-  if (seek_timeout_id == 0) {
-    seek_timeout_id =
-        g_timeout_add (SCRUB_TIME, (GSourceFunc) end_scrub, widget);
+    if (seek_timeout_id == 0) {
+      seek_timeout_id =
+          g_timeout_add (SCRUB_TIME, (GSourceFunc) end_scrub, widget);
+    }
   }
-#endif
 }
 
 static void
@@ -1122,8 +1125,13 @@ set_update_scale (gboolean active)
 static gboolean
 start_seek (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
 {
-  if (state == GST_STATE_PLAYING && flush_seek)
+  if (event->type != GDK_BUTTON_PRESS)
+    return FALSE;
+
+  if (state == GST_STATE_PLAYING && flush_seek) {
+    GST_DEBUG ("start scrub seek, PAUSE");
     gst_element_set_state (pipeline, GST_STATE_PAUSED);
+  }
 
   set_update_scale (FALSE);
 
@@ -1144,6 +1152,7 @@ stop_seek (GtkWidget * widget, gpointer user_data)
   }
 
   if (!flush_seek) {
+    GST_DEBUG ("do final seek");
     do_seek (widget);
   }
 
@@ -1152,10 +1161,11 @@ stop_seek (GtkWidget * widget, gpointer user_data)
     seek_timeout_id = 0;
     /* Still scrubbing, so the pipeline is already playing */
   } else {
-    if (state == GST_STATE_PLAYING)
+    if (state == GST_STATE_PLAYING) {
+      GST_DEBUG ("stop scrub seek, PLAYING");
       gst_element_set_state (pipeline, GST_STATE_PLAYING);
+    }
   }
-
   set_update_scale (TRUE);
 
   return FALSE;
@@ -1253,6 +1263,12 @@ flush_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
   flush_seek = gtk_toggle_button_get_active (button);
 }
 
+static void
+scrub_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
+{
+  scrub = gtk_toggle_button_get_active (button);
+}
+
 static void
 segment_done (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
 {
@@ -1343,6 +1359,7 @@ main (int argc, char **argv)
 {
   GtkWidget *window, *hbox, *vbox, *play_button, *pause_button, *stop_button;
   GtkWidget *accurate_checkbox, *key_checkbox, *loop_checkbox, *flush_checkbox;
+  GtkWidget *scrub_checkbox;
   GOptionEntry options[] = {
     {"stats", 's', 0, G_OPTION_ARG_NONE, &stats,
         "Show pad stats", NULL},
@@ -1396,6 +1413,8 @@ main (int argc, char **argv)
   key_checkbox = gtk_check_button_new_with_label ("Key_unit Seek");
   loop_checkbox = gtk_check_button_new_with_label ("Loop");
   flush_checkbox = gtk_check_button_new_with_label ("Flush");
+  scrub_checkbox = gtk_check_button_new_with_label ("Scrub");
+
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (flush_checkbox), TRUE);
 
   adjustment =
@@ -1422,6 +1441,7 @@ main (int argc, char **argv)
   gtk_box_pack_start (GTK_BOX (hbox), key_checkbox, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), loop_checkbox, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), flush_checkbox, FALSE, FALSE, 2);
+  gtk_box_pack_start (GTK_BOX (hbox), scrub_checkbox, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (vbox), hscale, TRUE, TRUE, 2);
 
   /* connect things ... */
@@ -1439,6 +1459,8 @@ main (int argc, char **argv)
       G_CALLBACK (loop_toggle_cb), pipeline);
   g_signal_connect (G_OBJECT (flush_checkbox), "toggled",
       G_CALLBACK (flush_toggle_cb), pipeline);
+  g_signal_connect (G_OBJECT (scrub_checkbox), "toggled",
+      G_CALLBACK (scrub_toggle_cb), pipeline);
 
   g_signal_connect (G_OBJECT (window), "destroy", gtk_main_quit, NULL);
 
@@ -1466,6 +1488,8 @@ main (int argc, char **argv)
         pipeline);
     g_signal_connect (bus, "message::tag", (GCallback) message_received,
         pipeline);
+    g_signal_connect (bus, "message::element", (GCallback) message_received,
+        pipeline);
     g_signal_connect (bus, "message::segment-done",
         (GCallback) message_received, pipeline);
     g_signal_connect (bus, "message::segment-done", (GCallback) segment_done,
